RUI_Version = "1.02.00"
RUI_ConfigVersion = "1.01.00"
--[[ Creates several heal and buff buttons for each member, similar to Benecast without overhealing options and the buttons can't disappear.

To Do:
1. Set Curse, Disease, Poison, Magic Icon Textures


-----------------------------------------------------------------------------------------------------------]]
RUI_Button_CurrentState = 0 -- Tracks current button states. The state is changed by modifier buttons (none, alt, ctrl, shift)

RUI_SpellOptionsDone = nil

RUI_Max_State = 3 -- The number of the highest state. Number of states are 0-3
RUI_Max_Units = 40 -- Max Number of Units to check if present.
RUI_Max_Buttons = 12 -- Max Number of buttons per UFrame
RUI_Max_UFrames = 40 -- Max Number of UFrames (Unit Frames) that can be shown

RUI_Max_Favs = 25
RUI_Num_Favs = 0
RUI_Fav_List = {}
RUI_Fav = {{{}}}

RUI_Num_UFrames = 1

-- Saved general settings
RUI_Settings = {}
RUI_PANEL = {{{}}}
RUI_Unit = {{{}}}
RUI_UFrame = {{{}}}

RUI_SpellState = {{{{}}}} 
RUI_Profile = {{{}}}

RUI_SpellOptions = {{}}
RUI_Menu_SpellOption = {}
RUI_SpellList = {}
RUI_MenuSpellOptions = {{}}
RUI_MenuSpellList = {}

-- Pending change flag. Since changes can't be made in combat a flag is needed to indicate make changes after combat.
RUI_SpellChange = false;
RUI_PartyChange = false;
RUI_InRaid = nil
local RUI_AuraChange = 1
local RUI_LastAura = GetTime()
local RUI_UNIT_AURA_Counter = 0
local RUI_UNIT_AURA_Counter_Expire = GetTime() + 60

RUI_FlipFlop = 1

-- MiniMap Button Settings
RUI_MMButton_Entry = {}
RUI_MMButton_NumEntries = 0

RUI_AddOn_Loaded = nil
-- Create Update timer variables
local RUI_Update_Timer_Repeating = nil
local RUI_Update_Timer_Duration = nil
local RUI_Update_Timer_StartTime = nil
local RUI_Update_Timer_EndTime = nil
local RUI_Update_Timer_Button = nil

local RUI_Update_Delay = GetTime()

-- Temp copy of general settings for menus.
RUI_MenuSettings = {}
-- Temp menu settings that are a duplicate of game settings.
RUI_MenuSpellState = {} -- 0=player, 1=Party, 2=Non-Party  (copy for temp menu settings)
RUI_Menu_StateNames = {}
RUI_Menu_ModeNames = {}


--[[ ===========================================================================================================================
					Event Handling Functions
=================================================================================================================================]]

-- Create a frame to register and receive events
local evFrame = CreateFrame("Frame");
-- Register Events
evFrame:RegisterEvent("SPELLS_CHANGED");
evFrame:RegisterEvent("ADDON_LOADED");
evFrame:RegisterEvent("PLAYER_REGEN_ENABLED");
evFrame:RegisterEvent("PARTY_MEMBERS_CHANGED");
evFrame:RegisterEvent('RAID_ROSTER_UPDATE');
evFrame:RegisterEvent("PLAYER_LOGIN");

function RUI_FrameOnEvent(self, event, arg1)
		
	if (event == "ADDON_LOADED" and arg1 == "RaidUI") then
		if not (RUI_Settings.SettingsLoaded) then
			RUI_Chat("SETTINGS NOT LOADED!!")
		else
			RUI_Chat("Settings were loaded")
		end
		RUI_Init_Nonsaved();
		if not (RUI_Settings.LoadedBefore) then
			RUI_Init_Settings();
			RUI_Chat("First time loaded. Setting Default Settings...");
		else
			RUI_Chat("Settings loaded...");
		end
		RUI_Chat("type /rui menu to configure Settings.");
		RUI_Init_Slash()
	end
		
	
	if (RUI_Update_Timer_Button) and (not RUI_Update_Timer_Button:IsVisible()) then
		RUI_Set_Update_Timer(tInterval)
	end
	
	
	if (event == "PLAYER_LOGIN") then
		RUI_SpellChange = true;
		if (RUI_Settings.UpdateInterval) then
			local tInterval = RUI_Settings.UpdateInterval/1000
			RUI_Set_Update_Timer(tInterval)
		end
    end
		
	if (event == "SPELLS_CHANGED") then
		RUI_SpellChange = true;
	elseif ( event == "PARTY_MEMBERS_CHANGED") or ( event == "RAID_ROSTER_UPDATE") then
		RUI_PartyChange = true;
	end
end

evFrame:SetScript("OnEvent", RUI_FrameOnEvent);

--[[ ===========================================================================================================================
					Update Timer Functions
	The update timer uses the OnUpdate of a dummy frame to check for elapsed time. OnUpdate for a frame is 
	called whenever the screen is redrawn. So I calculate what the time will be for the next update and then
	check if the current time is past that when ever onupdate is called. Once the elapse time is reach the
	status (grayouts, OOM, range) of each button is updated and the next update time is re-calculated.
=================================================================================================================================]]

function RUI_Create_Update_Timer_Frame()
	RUI_Update_Timer_Button = CreateFrame("Frame"); -- Create a dummy frame
	RUI_Update_Timer_Button:SetScript("OnUpdate", RUI_Update_Timer_OnUpdate) -- Set script so that RUI_Update_Timer_OnUpdate is called when ever there is an update on the frame.
end

function RUI_Set_Update_Timer(tInterval)
	if (not tInterval) or (tInterval<0) then return; end -- Verify a valid interval time was sent
	RUI_Update_Timer_Duration = tInterval -- Set the update timer values
	RUI_Update_Timer_StartTime = GetTime() -- GetTime returns time in seconds, so for example 100ms = 0.100 seconds.
	RUI_Update_Timer_EndTime = GetTime() + RUI_Update_Timer_Duration -- Calculate the time of next update
	RUI_Update_Timer_Button:Show(); -- Show frame so that is will start updating. The frame has no graphics so it will not be visible
end

function RUI_Update_Timer_OnUpdate()
	-- Check if the current time is past the calculated next update time
	if (not RUI_Update_Timer_EndTime) or (GetTime()<RUI_Update_Timer_EndTime) then return; end
	
	if (RUI_FlipFlop) then 
		RUI_FlipFlop = nil
	else
		RUI_FlipFlop = 1
	end
	
	if (RUI_Party_Frame:IsShown()) then 
		RUI_Refresh_UFrames() 	-- Update the status (grayouts, OOM, heal indicators, and range) of all buttons
	end
	
	if (RUI_Settings.FavsEnable) and (RUI_Favorites_Frame:IsShown()) then
		RUI_Refresh_Favs()
	end
	
	-- There is a change pending, if not in combat then configure buttons
    if (not InCombatLockdown()) then
		if (RUI_SpellChange) or (RUI_PartyChange) then 
			if (GetTime()>RUI_Update_Delay) then
				RUI_Config_Panels();
				RUI_Update_Buttons()
				RUI_Fav_Config_Panel()
				RUI_Fav_Update_Buttons()
				if (RUI_Settings.FavsEnable) then
					RUI_Fav_Header_Frame:Show()
				else
					RUI_Fav_Header_Frame:Hide()
				end
				
				RUI_Menu_Create_SpellSelect_Frame()
				RUI_SpellChange = false;
				RUI_PartyChange = false;
				RUI_Fav_Change = nil
				RUI_Update_Delay = GetTime() + 5
				--Reset the update timer with possible interval changes
				if (RUI_Settings.UpdateInterval) then
					-- Update Interval is entered as ms, but the timer function uses seconds so ms must be converted to seconds.
					local tInterval = RUI_Settings.UpdateInterval/1000
					RUI_Set_Update_Timer(tInterval)
				end
			end
		else
			if (RUI_Fav_Change) then
				RUI_Fav_Config_Panel()
				RUI_Fav_Update_Buttons()
				RUI_Fav_Change = nil
			end
		end
		
    end
		
	-- Re-calculate the next update time. The timer will continuelly run until the dummy frame is hidden.
	RUI_Update_Timer_EndTime = GetTime() + RUI_Update_Timer_Duration
	RUI_Update_Timer_StartTime = GetTime()
end
