--[[ ===========================================================================================================================
					Button and Header Functions
=================================================================================================================================]]
local RUI_Unit_Frame_Width = 100
local RUI_Unit_Frame_Height = 36

function RUI_Hide_Panel()
	if (RUI_Party_Frame) then -- Check if the header exist, if it exists then hide it.
		RUI_Party_Frame:SetAlpha(0);
		RUI_Party_Frame:Hide();
	end
end

function RUI_Show_Panel()
	if (RUI_Party_Frame) then -- Check if the header exist, if it exists then hide it.
		RUI_Party_Frame:SetAlpha(1);
		RUI_Party_Frame:Show();
	end
end

function RUI_Set_Width()
	local tParent = getglobal("RUI_Party_Frame");	
	tParent:SetWidth(RUI_Unit_Frame_Width)
end

--[[ ===========================================================================================================================

					CREATE FRAME AND BUTTON FUNCTIONS

					
=================================================================================================================================]]

--[[ ===========================================================================================================================
					Create Main Panel Functions
=================================================================================================================================]]

--=================================== CONFIGURE BUTTONS =========================================================================
function RUI_Config_Panels()
	-- Clear Current Values of UFrames
	for k= 1, RUI_Max_UFrames do
		RUI_UFrame[k].UnitNumber = nil -- Which RUI_Unit this UFrame is showing
		RUI_UFrame[k].Show = nil -- Whether the UFrame should be shown
		RUI_UFrame[k].LastHealth = 0 -- Last checked Health value, used to decide whether to update healthbar.
	end
	-------- Set Panel Settings -----------------------
	RUI_Header_Frame:Show();
	RUI_Header_Frame:SetToplevel(1)	
	local tPBackdrop = getglobal(RUI_Party_Frame:GetName() .. "Backdrop");
	if (tPBackdrop) then
		tPBackdrop:SetAlpha(0.75);
	end
	-- Get Party Num and Info
	local RUI_InRaid = nil
	if (IsInRaid()) then
		RUI2_InRaid = 1
	else
		RUI2_InRaid = nil
	end
	if (RUI_InRaid) and (RUI_Settings.Raid) then
		RUI_SetRaidUnits()
		RUI_Num_UFrames = GetNumGroupMembers();
	else
		RUI_SetPartyUnits()
		RUI_Num_UFrames = GetNumGroupMembers() + 1;
	end
	if (RUI_Num_UFrames>RUI_Max_UFrames) then
		RUI_Num_UFrames = RUI_Max_UFrames
	end
	
	
	------------- Calculate Unit Frame Height and Width -------------
	local tPadding = 175 -- Space for health bar
	RUI_Unit_Frame_Width = ((36 + 5)*RUI_Settings.NumberButtons)+tPadding + 4  -- Calculate Width of UFrames
	RUI_Unit_Frame_Height = 36 + 4 -- Calculate Unit Frame Height
	RUI_Party_Frame:SetScale(1);
	local tParent = getglobal("RUI_Party_Frame");
	tParent:SetWidth(RUI_Unit_Frame_Width)
	RUI_Party_Frame:SetScale(RUI_Settings.Scale);
	
	local UFrame_Count = 0;
	if (RUI_InRaid) then --and () then
		for k=0, RUI_Num_UFrames-1 do --RUI_Max_Units do
			RUI_UFrame[k].UnitNumber = k -- Which RUI_Unit this UFrame is showing
			RUI_UFrame[k].Show = 1 -- Whether the UFrame should be shown
			RUI_UFrame[k].LastHealth = 0 -- Last checked Health value, used to decide whether to update healthbar.	
			RUI_Config_UFrame(k)
			UFrame_Count = k
		end
	else
		for k=0, RUI_Num_UFrames-1 do			
			RUI_UFrame[k].UnitNumber = k -- Which RUI_Unit this UFrame is showing
			RUI_UFrame[k].Show = 1 -- Whether the UFrame should be shown
			RUI_UFrame[k].LastHealth = 0 -- Last checked Health value, used to decide whether to update healthbar.
			RUI_Config_UFrame(k)
			UFrame_Count = k
		end
	end
	
	for f = 0, RUI_Max_UFrames do
		local hdr = RUI_UFrame[f].Header;
		if (hdr) then
			if (RUI_UFrame[f].Show) then 
				hdr:Show();
			else
				hdr:Hide();
			end
		end
	end
	RUI_Num_UFrames = UFrame_Count;

	if (RUI_Num_UFrames<RUI_Settings.MaxRows) then
		tParent:SetHeight(RUI_Unit_Frame_Height * (RUI_Num_UFrames+1) + 10)
	else
		tParent:SetHeight(RUI_Unit_Frame_Height * RUI_Settings.MaxRows + 30)
	end	
	local tColumns = (floor((RUI_Num_UFrames)/RUI_Settings.MaxRows)) + 1
	tParent:SetWidth(RUI_Unit_Frame_Width * tColumns + 10)
		
	if (RUI_Settings.HidePanel) then
		RUI_Hide_Panel()
	else
		RUI_Show_Panel()
	end
end

--[[ ===========================================================================================================================
					Create UFrame Functions
=================================================================================================================================]]

--------------------------------CONFIG UNIT FRAME--------------------------------------------------------
function RUI_Config_UFrame(tUFrame)
	if not (tUFrame) then return; end
	
	local tParent = getglobal("RUI_Party_Frame");
	local tButton_Size = 36
	local tButton_Spacing = 5
	------ CONFIGURE HEADER --------------
	
	local hdr = RUI_UFrame[tUFrame].Header; -- Fetch header handle
	-- Check if the header exist, if not create the header
	if (not hdr) then
		-- Create frame based on secure header template declared in RaidUI.xml
		hdr = CreateFrame("Frame", "RUI_Unit_Header"..tUFrame, tParent, "RUI_Unit_Template"); 
		hdr:SetParent("RUI_Party_Frame");
		hndlr = CreateFrame("Frame", hdr:GetName().."_SucHdr", hdr, "SecureHandlerStateTemplate")
		RUI_UFrame[tUFrame].Header = hdr;
		RUI_UFrame[tUFrame].Handler = hndlr;
	end
	
	hdr:SetParent("RUI_Party_Frame");
			
	local tVoffset = 0
	local tLabel = getglobal(hdr:GetName() .. "_Label");
	if (tLabel) then
		tLabel:Hide();
	end
		
	local tColumn = floor((tUFrame)/RUI_Settings.MaxRows)
	local tRow = tUFrame - (tColumn * RUI_Settings.MaxRows)
	hdr:SetPoint("TOPLEFT", tParent, "TOPLEFT", (tColumn * RUI_Unit_Frame_Width), (-1*tRow*RUI_Unit_Frame_Height)-5);
		
	local cOverlay = getglobal(hdr:GetName() .. "_HealthBar_ClickOverlay");
	--local fString = getglobal(hdr:GetName() .. "_HealthBar_Text1");
	--local hBar = getglobal(hdr:GetName() .. "_HealthBar");
	--hBar:SetToplevel(0)
	cOverlay:SetFrameLevel(hdr:GetFrameLevel() + 5);
	--cOverlay:SetToplevel(1)
	cOverlay:SetFrameStrata("TOOLTIP")

	-- Set frame's height and width to fit number of buttons and name label
	hdr:SetWidth(RUI_Unit_Frame_Width);
	hdr:SetHeight(RUI_Unit_Frame_Height);
			
	local tHB_XPos = 10
	-- Set the Name text on healthbar
	local tHB_Name_Text = getglobal(hdr:GetName() .. "_HealthBar_Text1");
	if (tHB_Name_Text) then 
		-- Set the frame label to target unit's full name
		local tName = UnitName(RUI_Unit[RUI_UFrame[tUFrame].UnitNumber].Unit)
		if (tName) then
			tHB_Name_Text:SetText(tName)
			tHB_Name_Text:Show();
		else -- Don't show name test. Set it to blank and hide it.
			tHB_Name_Text:SetText("")
			tHB_Name_Text:Hide();
		end
		local HB = getglobal(hdr:GetName() .. "_HealthBar");
		HB:Show();
		HB:SetFrameLevel(hdr:GetFrameLevel() + 1);
		HB:SetValue(0)
		RUI_UFrame[tUFrame].LastHealth = 0
		
		local RUI_HB_Height = tButton_Size - 4 -- padding above HB(2) and below HB(2)
		HB:SetHeight(RUI_HB_Height)
		HB:SetPoint("LEFT", hdr, "LEFT", tHB_XPos, tVoffset);
	end
	-- Clear any key bindings to buttons
	ClearOverrideBindings(hdr);
	-- CONFIGURE SPELL BUTTONS ---------------
	local RUI_Button_Anchor = HB
	for i = 1, RUI_Max_Buttons do -- Cycle through all RUI_Max_Buttons possible buttons
		RUI_Config_Button(tUFrame, i, RUI_Button_Anchor)
		RUI_Button_Anchor = RUI_UFrame[tUFrame].Button[i]
	end -- End for i loop (cycle through buttons)
	
	if (RUI_UFrame[tUFrame].Show) then 
		hdr:Show();
	else
		hdr:Hide();
	end
	-- Force a state refresh to update everthing
	hndlr:SetAttribute("state", RUI_Button_CurrentState);
end

