function RUI_Init_Menu_Values()
	-- Initialize Menu Variables. These are same as game variables. They are used to store values while editing menu. Then can be copied to game settings on OK or ignored on Cancel.
	RUI_Menu_StateNames[0] = "Normal"
	RUI_Menu_StateNames[1] = "<Alt>"
	RUI_Menu_StateNames[2] = "<Ctrl>"
	RUI_Menu_StateNames[3] = "<Shift>"
	-- Create and Init Menu variables
	-- These values are used to store menu changes so that if cancel is pressed they can be ignored and no changes are done to game settings
	-- , or if ok is pressed they are copy to the game settings.
	-- Settings
	RUI_MenuSettings.Enable = 1
	RUI_MenuSettings.LockPanel = nil
	RUI_MenuSettings.UpdateInterval = 400
	RUI_MenuSettings.Scale = 0.60
	RUI_MenuSettings.TooltipEnable = nil
	RUI_MenuSettings.SimpleTooltip = 1
	RUI_MenuSettings.NumberButtons = 4
	RUI_MenuSettings.Raid = nil
	RUI_MenuSettings.MaxRaidFrames = 25
	RUI_MenuSettings.MaxRows = 15
	RUI_MenuSettings.CheckRange = 1
	RUI_MenuSettings.RangeSpellName = nil
	RUI_MenuSettings.DisableModifiers = nil
	
	RUI_MenuSettings.FavsEnable = 1
		
	RUI_MenuSpellState={}
	RUI_MenuSpellState.NumButtons = 0
				
	for s=0, RUI_Max_State do	-- Button States. State are the different spell sets depending on what modifier key is pressed (0=none,1=alt,2=ctrl,3=shift)
		RUI_MenuSpellState[s]={}
		RUI_MenuSpellState[s].Button={}
		for b = 1, RUI_Max_Buttons do -- Buttons. Can have up to RUI_Max_Buttons buttons per bar.
			RUI_MenuSpellState[s].Button[b] = {}
			RUI_MenuSpellState[s].Button[b].SpellName = nil
		end
	end
		
	if not (RUI_SpellOptionsDone) then
		--RUI_Chat("Clearing SpellOptions")
		for i=1, 40 do
			RUI_Menu_SpellOption[i] = nil
		end
		RUI_Menu_Create_SpellSelect_Frame()
		RUI_SpellOptionsDone = 1
	end
end

function RUI_Init_UFrame_Values()
	
	for s=0, RUI_Max_UFrames do
		RUI_UFrame[s] = {}
		RUI_UFrame[s].UnitNumber = nil -- Which RUI_Unit this UFrame is showing
		RUI_UFrame[s].Show = nil -- Whether the UFrame should be shown
		RUI_UFrame[s].IsFavorite = nil
	
		RUI_UFrame[s].Header = nil -- The header of the Unit Frame
		RUI_UFrame[s].Handler = nil
		RUI_UFrame[s].Button = {}
		RUI_UFrame[s].Button[0] = nil
		RUI_UFrame[s].IsActive = {}
		RUI_UFrame[s].HasDebuff = {}
		RUI_UFrame[s].LastExpTime = {}
		
		RUI_UFrame[s].HasCurse = nil
		RUI_UFrame[s].HasDisease = nil
		RUI_UFrame[s].HasPoison = nil
		RUI_UFrame[s].HasMagic = nil
		RUI_UFrame[s].HasWS = nil
		
		RUI_UFrame[s].IsDead = nil
		
		for t = 0, RUI_Max_Buttons do
			RUI_UFrame[s].Button[t] = nil -- Create reference and default value to nil for Buttons of this UFrame
			RUI_UFrame[s].IsActive[t] = nil-- Tracks whether the mod thinks the button is active. So can clear cooldown if it isn't
			RUI_UFrame[s].HasDebuff[t] = nil
			RUI_UFrame[s].LastExpTime[t] = nil
		end
		RUI_UFrame[s].LastHealth = 0 -- Last checked Health value, used to decide whether to update healthbar.
	end
end

function RUI_Init_Unit_Values()
-- Create and Init panel settings
	for s=0, RUI_Max_Units do
		RUI_Unit[s] = {}
		RUI_Unit[s].Enable = nil -- Whether the unit should be shown, if present
		RUI_Unit[s].Unit = "none" -- The unit the spells will target
	end
	-- The unit each panel will target it's spell on
	RUI_Unit[0].Unit = "Player"
	RUI_Unit[1].Unit = "Party1"
	RUI_Unit[2].Unit = "Party2"
	RUI_Unit[3].Unit = "Party3"
	RUI_Unit[4].Unit = "Party4"
	RUI_Unit[5].Unit = "pet"  --'pet', 'partypet1', 'partypet2', 'partypet3', 'partypet4'
	RUI_Unit[6].Unit = "partypet1"
	RUI_Unit[7].Unit = "partypet2"
	RUI_Unit[8].Unit = "partypet3"
	RUI_Unit[9].Unit = "partypet4"
	
end


-- This function intializes all non-saved data.
function RUI_Init_Nonsaved()
	RUI_PANEL.Header = nil -- Stores the header frame created
	RUI_PANEL.AttachToUnit = "UIParent" -- The frame the bar will be attached to (such as playerframe)
	
	RUI_Init_Unit_Values();
	RUI_Init_UFrame_Values()
	RUI_Init_Fav_Values();
	RUI_Init_Menu_Values();
	
	
	-- Check if the last saved setting's config version is out of date.
	if (RUI_Settings.ConfigVersion) then 
		RUI_Chat("Config Version="..RUI_Settings.ConfigVersion)
		-- Saved config version is old, try to update without losing data.
		if (RUI_Settings.ConfigVersion~=RUI_ConfigVersion) then
			if (RUI_Settings.ConfigVersion==3.01) then
				RUI_Chat("Saved configuration is outdated. Updating configuration, no saved values will be lost.");
				RUI_Update_Settings301()
			elseif (RUI_Settings.ConfigVersion==3.02) then
				RUI_Chat("Saved configuration is outdated. Updating configuration, no saved values will be lost.");
				RUI_Update_Settings302() 
			else
				-- Unable to update settings, have to re-initialize settings and lose saved data.
				RUI_Chat("Saved configuration version Outdated. Initializing saved values.");
				RUI_Init_Settings()
			end
		end
	else
		-- No saved config version. Have to re-initialize settings and lose saved data.
		RUI_Chat("Saved configure version corrupted. Initializing saved values.");
		RUI_Init_Settings()
	end
	-- Create Update Timer frame to prepare the Update Timer.
	RUI_Create_Update_Timer_Frame()
		
	local tClass = (select(2,UnitClass("player")))
	if (tClass) then
		if (tClass == "PALADIN") then
			RUI_Paladin_SetDefaultSpellOptions()
		elseif (tClass == "PRIEST") then
			RUI_Priest_SetDefaultSpellsOptions()
		elseif (tClass == "DRUID") then
			RUI_Druid_SetDefaultSpellOptions()
		elseif (tClass == "SHAMAN") then
			RUI_Shaman_SetDefaultSpellOptions()
		elseif (tClass == "MONK") then
			RUI_Monk_SetDefaultSpellsOptions()
		end
	end
		
	RUI_Settings.SettingsLoaded = 1
	
	--RUI_Profiles_Init()
end

-- This function initializes all saved vairiables. All spells and settings are wiped.
function RUI_Init_Settings()
	RUI_Settings.LoadedBefore = 1
	RUI_Settings.Version = RUI_Version
	RUI_Settings.ConfigVersion = RUI_ConfigVersion
	RUI_Settings.LockPanel = nil
	RUI_Settings.UpdateInterval = 200
	RUI_Settings.Scale = 0.60
	
	RUI_Settings.TooltipEnable = nil
	RUI_Settings.SimpleTooltip = 1
	RUI_Settings.NumberButtons = 4
	RUI_Settings.Raid = 1
	RUI_Settings.MaxRaidFrames = 25
	RUI_Settings.MaxRows = 15
	RUI_Settings.CheckRange = 1
	RUI_Settings.RangeSpellName = nil
	RUI_Settings.DisableModifiers = 1
	
	RUI_Settings.HidePanel = nil
	RUI_Settings.HideFavPanel = nil
	
	RUI_Settings.FavsEnable = nil
			
	RUI_Settings.MenuUnit = 1
	RUI_Settings.MenuTab = 1
	RUI_Settings.MenuState = 0
		
	RUI_Settings.Debug = nil

	RUI_SpellState={}
	RUI_SpellState.NumButtons = 0
	for s=0, RUI_Max_State do	-- Button States. State are the different spell sets depending on what modifier key is pressed (0=none,1=alt,2=ctrl,3=shift)
		RUI_SpellState[s]={}
		RUI_SpellState[s].Button={}
		for b = 1, RUI_Max_Buttons do -- Buttons. Can have up to RUI_Max_Buttons buttons per bar.
			RUI_SpellState[s].Button[b]={}
			RUI_SpellState[s].Button[b].SpellName = nil
		end
	end
		

	local tClass = (select(2,UnitClass("player")))
	if (tClass) then
		if (tClass == "PALADIN") then
			RUI_PaladinDefault()
			RUI_Paladin_SetDefaultSpellOptions()
		elseif (tClass == "PRIEST") then
			RUI_PriestDefault()
			RUI_Priest_SetDefaultSpellsOptions()
		elseif (tClass == "DRUID") then
			RUI_DruidDefault()
			RUI_Druid_SetDefaultSpellOptions()
		elseif (tClass == "SHAMAN") then
			RUI_ShamanDefault()
			RUI_Shaman_SetDefaultSpellOptions()
		elseif (tClass == "MONK") then
			RUI_MonkDefault()
			RUI_Monk_SetDefaultSpellOptions()
		end
	end
		
	RUI_Profiles_Init()
end

-- Update Settings Functions will update older version of RaidUI without loosing any saved data. 
-- Each function updates to the next version, and the verision update functions are chained to update mods that are more than one version behind.
--updates config to version 0.02
function RUI_Update_Settings301() 
	RUI_Settings.Version = RUI_Version
	RUI_Settings.ConfigVersion = RUI_ConfigVersion
	-- insert setting updates to go from config version 1 to 2.
	RUI_Update_Settings302()
end

function RUI_Update_Settings302() 
	RUI_Settings.Version = RUI_Version
	RUI_Settings.ConfigVersion = RUI_ConfigVersion
	RUI_Profiles_Init()
end


